
/*
This document is about how the module goes from passthrough mode into AT command mode and back to passthrough mode. 
This document is for USR-WIFI232(-T-G-S-H) low power module.
note：contains pseudo-code，you should modify it for your application.
*/


#define MAX_RX_BUF_LEN 100 //the max receive buffer length, you might change it for your application

char mode; //record current mode status, 1 for the command mode, 0 for the passthrough mode
char uart_rx_buf[MAX_RX_BUF_LEN]; //the receive buffer of uart
int uart_rx_cnt; //the counter of data received
void DelayMS( int ms ); //delay function in millisecond
void UartSendChar( char chr ); //send one character by uart
vodi UartSendString( char* str ); //send a string by uart

//get module into AT command mode
int Module2CMD()
{
	if( !mode ) //confirm the module is not in the command mode
	{
		uart_rx_cnt = 0; //reset the counter of receive buffer
		memset( uart_rx_buf, 0, MAX_RX_BUF_LEN ); //clean the receive buffer
		DelayMS(50); //there is no data in 50 ms
		UartSendChar('+'); //send '+' to module
		DelayMS(10); //wait 10 ms
		UartSendChar('+');
		DelayMS(10);
		UartSendChar('+');
		DelayMS(400);
		
		if ( 'a' == uart_rx_buf[0] ) //receive 'a'
		{
			UartSendChar('a'); //send 'a'
			DelayMS(10);
			return 1;
		}
		else if ( '+' == uart_rx_buf[0] && '+' == uart_rx_buf[1] && '+' == uart_rx_buf[2] ) //already in command mode with echo
		{
			return 1;
		}
		else return 0;
	}
	else return 1;
}

//get module back to passthrough mode
int Module2Pass()
{
	if( mode ) //confirm the module is in the command mode
	{
		uart_rx_cnt = 0; //reset the counter of receive buffer
		memset( uart_rx_buf, 0, MAX_RX_BUF_LEN ); //clean the receive buffer
		UartSendString("AT+ENTM\r"); //send command "ENTM" to get module out of AT command mode
		DelayMS(200); //wait 200 ms to receive the response from module
		if ( uart_rx_cnt >= 7 && 'o' == uart_rx_buf[1] && 'k' == uart_rx_buf[2] ) return 1;
		else if ( uart_rx_cnt >= 16 && 'o' == uart_rx_buf[10] && 'k' == uart_rx_buf[11] ) return 1;
		return 0;
	}
	else return 1;
}


int main()
{
	int i;

	//.........

	//here we get the module into AT command mode
	while ( the nReady pin is high ) nop; //waiting until the nReady pin is low
	i = 3; //try 3 times
	while ( i )
	{
		if ( Module2CMD() ) break;
		else i--;
	}

	if ( i > 0 ) //here you can check i, if i>0 module has been in the command mode
	{
		//send commands to module, for example:get module's version
		uart_rx_cnt = 0; //reset the counter of receive buffer
		memset( uart_rx_buf, 0, MAX_RX_BUF_LEN ); //clean the receive buffer
		UartSendString("AT+VER\r");
		//if echo is unvalid, the "uart_rx_buf" buffer might be "+ok=V1.0.04a\r\n"
		//if echo is valid, the "uart_rx_buf" buffer might be "AT+VER\n\r+ok=V1.0.04a\r\n"
	}
	else
	{
		//here you can do fault-tolerance processing
	}

	//here we get the module out of AT command mode
	while ( the nReady pin is high ) nop; //waiting until the nReady pin is low
	i = 3; //try 3 times
	while ( i )
	{
		if ( Module2Pass() ) break;
		else i--;
	}

	if ( i <= 0 ) //here you can check i, if i<=0 module has not been in the passthrough mode
	{
		//here you can do fault-tolerance processing
	}

	//.........
}